@@ -12,9 +12,11 @@ class Field(object):
     
     :param name: name of field
     :param store, index, termvector: field parameters, expressed as bools or strs, with lucene defaults
+    :param attrs: additional attributes to set on the field
     """
-    def __init__(self, name, store=False, index='analyzed', termvector=False):
+    def __init__(self, name, store=False, index='analyzed', termvector=False, **attrs):
         self.name = name
+        self.attrs = attrs
         if isinstance(store, bool):
             store = 'yes' if store else 'no'
         if isinstance(index, bool):
@@ -26,7 +28,13 @@ class Field(object):
     def items(self, *values):
         "Generate lucene Fields suitable for adding to a document."
         for value in values:
-            yield lucene.Field(self.name, value, self.store, self.index, self.termvector)
+            try:
+                field = lucene.Field(self.name, value, self.store, self.index, self.termvector)
+            except lucene.InvalidArgsError:
+                field = lucene.Field(self.name, value, self.termvector)
+            for name, value in self.attrs.items():
+                setattr(field, name, value)
+            yield field
 
 class FormatField(Field):
     """Field which uses string formatting on its values.
@@ -40,6 +48,36 @@ class FormatField(Field):
         "Generate fields with formatted values."
         return Field.items(self, *map(self.format, values))
 
+class NumericField(Field):
+    """Field which indexes numbers in a prefix tree.
+    
+    :param name: name of field
+    :param step: precision step
+    """
+    def __init__(self, name, step=None, store=False, index=True):
+        Field.__init__(self, name, store)
+        self.step = step or lucene.NumericUtils.PRECISION_STEP_DEFAULT
+        self.index = index
+    def items(self, *values):
+        "Generate lucene NumericFields suitable for adding to a document."
+        for value in values:
+            field = lucene.NumericField(self.name, self.step, self.store, self.index)
+            if isinstance(value, float):
+                field.doubleValue = value
+            else:
+                field.longValue = long(value)
+            yield field
+    def range(self, start, stop, lower=True, upper=False):
+        "Return lucene NumericRangeQuery."
+        if isinstance(start, float) or isinstance(stop, float):
+            start, stop = (value if value is None else lucene.Double(value) for value in (start, stop))
+            return lucene.NumericRangeQuery.newDoubleRange(self.name, start, stop, lower, upper)
+        if start is not None:
+            start = None if start < lucene.Long.MIN_VALUE else lucene.Long(long(start))
+        if stop is not None:
+            stop = None if start > lucene.Long.MAX_VALUE else lucene.Long(long(stop))
+        return lucene.NumericRangeQuery.newLongRange(self.name, start, stop, lower, upper)
+
 class PrefixField(Field):
     """Field which indexes every prefix of a value into a separate component field.
     The customizable component field names are expressed as slices.
@@ -143,18 +181,26 @@ class DateTimeField(PrefixField):
         May produce invalid dates, but the query is still correct.
         """
         dates = (map(float, self.split(str(date))) for date in (start, stop))
-        queries = []
+        items = []
         for dates in self._range(*dates):
-            begin, end = (self.join(map('{0:02n}'.format, date)) for date in dates)
-            queries.append(PrefixField.range(self, begin, end))
-        queries[0] = PrefixField.range(self, str(start), queries[0].upperVal, lower=lower)
-        queries[-1] = PrefixField.range(self, queries[-1].lowerVal, str(stop), upper=upper)
+            items.append(tuple(self.join(map('{0:02n}'.format, date)) for date in dates))
+        queries = [PrefixField.range(self, str(start), items[0][1], lower=lower)]
+        queries += [PrefixField.range(self, *item) for item in items[1:-1]]
+        queries.append(PrefixField.range(self, items[-1][0], str(stop), upper=upper))
         return Query.any(*queries)
-    def within(self, days=0, seconds=0):
-        "Return date range query within current time and delta."
-        now = datetime.date.today() if (isinstance(days, int) and not seconds) else datetime.datetime.now()
-        delta = now + datetime.timedelta(days, seconds)
-        return self.range(*sorted([now, delta]), upper=True)
+    def within(self, days=0, weeks=0, utc=False, **delta):
+        """Return date range query within current time and delta.
+        If the delta is an exact number of days, then dates will be used.
+        
+        :param days, weeks: number of days to offset from today
+        :param utc: optionally use utc instead of local time
+        :params delta: additional timedelta parameters
+        """
+        now = datetime.datetime.utcnow() if utc else datetime.datetime.now()
+        if not (isinstance(days + weeks, float) or delta):
+            now = now.date()
+        delta = datetime.timedelta(days, weeks=weeks, **delta)
+        return self.range(*sorted([now, now + delta]), upper=True)
 
 class Document(object):
     """Delegated lucene Document.
@@ -195,6 +241,8 @@ class Document(object):
         self.doc.removeFields(name)
     def getlist(self, name):
         "Return list of all values for given field."
+        if lucene.VERSION <= '2.4.1': # memory leak in string array
+            return [field.stringValue() for field in self.doc.getFields(name)]
         return list(self.doc.getValues(name))
     def dict(self, *names, **defaults):
         """Return dict representation of document.
