@@ -7,6 +7,7 @@ The final `Indexer`_ classes exposes a high-level Searcher and Writer.
 import itertools, operator
 import contextlib
 import abc, collections
+import warnings
 import lucene
 from .queries import Query, HitCollector, Filter
 from .documents import Field, Document, Hits
@@ -18,17 +19,18 @@ class Atomic(object):
     def __subclasshook__(cls, other):
         return not issubclass(other, collections.Iterable) or NotImplemented
 Atomic.register(basestring)
+Atomic.register(lucene.TokenStream)
 
-def iterate(jit, positioned=False):
-    """Transform java iterator into python iterator.
-    
-    :param positioned: current iterator position is valid
-    """
-    with contextlib.closing(jit):
-        if positioned:
-            yield jit
-        while jit.next():
-            yield jit
+class Analyzer(lucene.PythonAnalyzer):
+    "Return a lucene Analyzer which chains together a tokenizer (or analyzer) and filters."
+    def __init__(self, tokenizer, *filters):
+        lucene.PythonAnalyzer.__init__(self)
+        self.tokenizer, self.filters = tokenizer, filters
+    def tokenStream(self, field, reader):
+        tokens = self.tokenizer.tokenStream(field, reader) if isinstance(self.tokenizer, lucene.Analyzer) else self.tokenizer(reader)
+        for filter in self.filters:
+            tokens = filter(tokens)
+        return tokens
 
 class IndexReader(object):
     """Delegated lucene IndexReader, with a mapping interface of ids to document objects.
@@ -47,14 +49,8 @@ class IndexReader(object):
         return 0 <= id < self.maxDoc() and not self.isDeleted(id)
     def __iter__(self):
         return itertools.ifilterfalse(self.isDeleted, xrange(self.maxDoc()))
-    def doc(self, id):
-        return self.indexReader.document(id)
     def __getitem__(self, id):
-        try:
-            doc = self.doc(id)
-        except lucene.JavaError:
-            raise KeyError(id)
-        return Document(doc)
+        return Document(self.document(id))
     def __delitem__(self, id):
         self.deleteDocument(id)
     @property
@@ -63,7 +59,6 @@ class IndexReader(object):
         return self.indexReader.directory()
     def delete(self, name, value):
         """Delete documents with given term.
-        
         Acquires a write lock.  Deleting from an `IndexWriter`_ is encouraged instead.
         """
         self.deleteDocuments(lucene.Term(name, value))
@@ -76,24 +71,32 @@ class IndexReader(object):
         return list(self.getFieldNames(option))
     def terms(self, name, start='', stop=None, counts=False):
         "Generate a slice of term values, optionally with frequency counts."
-        for termenum in iterate(self.indexReader.terms(lucene.Term(name, start)), positioned=True):
+        with contextlib.closing(self.indexReader.terms(lucene.Term(name, start))) as termenum:
             term = termenum.term()
-            if term and term.field() == name:
+            while term and term.field() == name:
                 text = term.text()
-                if stop is None or text < stop:
-                    yield (text, termenum.docFreq()) if counts else text
-                    continue
-            break
+                if stop is not None and text >= stop:
+                    break
+                yield (text, termenum.docFreq()) if counts else text
+                term = termenum.next() and termenum.term()
     def docs(self, name, value, counts=False):
         "Generate doc ids which contain given term, optionally with frequency counts."
-        for termdocs in iterate(self.termDocs(lucene.Term(name, value))):
-            doc = termdocs.doc()
-            yield (doc, termdocs.freq()) if counts else doc
-    def positions(self, name, value):
-        "Generate doc ids which contain given term, with their positions."
-        for termpositions in iterate(self.termPositions(lucene.Term(name, value))):
-            positions = [termpositions.nextPosition() for n in xrange(termpositions.freq())]
-            yield termpositions.doc(), positions
+        with contextlib.closing(self.termDocs(lucene.Term(name, value))) as termdocs:
+            while termdocs.next():
+                doc = termdocs.doc()
+                yield (doc, termdocs.freq()) if counts else doc
+    def positions(self, name, value, payloads=False):
+        "Generate doc ids and positions which contain given term, optionally only with payloads."
+        with contextlib.closing(self.termPositions(lucene.Term(name, value))) as termpositions:
+            while termpositions.next():
+                doc = termpositions.doc()
+                positions = (termpositions.nextPosition() for n in xrange(termpositions.freq()))
+                if payloads:
+                    array = lucene.JArray_byte('')
+                    yield doc, [(position, ''.join(termpositions.getPayload(array, 0))) \
+                        for position in positions if termpositions.payloadAvailable]
+                else:
+                    yield doc, list(positions)
     def comparator(self, name, *names, **kwargs):
         """Return sequence of documents' field values suitable for sorting.
         
@@ -111,16 +114,49 @@ class IndexReader(object):
                 while termdocs.next():
                     values[termdocs.doc()] = value
         return values
+    def spans(self, query, positions=False):
+        """Generate docs with occurrence counts for a span query.
+        
+        :param query: lucene SpanQuery
+        :param positions: optionally include slice positions instead of counts
+        """
+        spans = query.getSpans(self.indexReader)
+        spans = itertools.takewhile(lucene.Spans.next, itertools.repeat(spans))
+        for doc, spans in itertools.groupby(spans, key=lucene.Spans.doc):
+            if positions:
+                yield doc, [(span.start(), span.end()) for span in spans]
+            else:
+                yield doc, sum(1 for span in spans)
+    def termvector(self, id, field, counts=False):
+        "Generate terms for given doc id and field, optionally with frequency counts."
+        if lucene.VERSION <= '2.4.1':
+            warnings.warn('TermFreqVector.terms leaks memory')
+        tfv = self.getTermFreqVector(id, field)
+        return itertools.izip(tfv.terms, tfv.termFrequencies) if counts else iter(tfv.terms)
+    def positionvector(self, id, field, offsets=False):
+        "Generate terms and positions for given doc id and field, optionally with character offsets."
+        if lucene.VERSION <= '2.4.1':
+            warnings.warn('TermPositionVector.terms leaks memory')
+        tpv = lucene.TermPositionVector.cast_(self.getTermFreqVector(id, field))
+        for index, term in enumerate(tpv.terms):
+            if offsets:
+                yield term, map(operator.attrgetter('startOffset', 'endOffset'), tpv.getOffsets(index))
+            else:
+                yield term, list(tpv.getTermPositions(index))
 
 class Searcher(object):
     "Mixin interface common among searchers."
-    __getitem__ = IndexReader.__getitem__.im_func
-    def __init__(self, arg, analyzer=lucene.StandardAnalyzer):
+    def __init__(self, arg, analyzer=None):
         super(Searcher, self).__init__(arg)
-        self.analyzer = analyzer()
+        if not isinstance(analyzer, (lucene.Analyzer, type(None))):
+            warnings.warn("Support for analyzer constructors will be removed.", DeprecationWarning)
+            analyzer = analyzer()
+        self.analyzer = lucene.StandardAnalyzer() if analyzer is None else analyzer
     def __del__(self):
         if str(self) != '<null>':
             self.close()
+    def __getitem__(self, id):
+        return Document(self.doc(id))
     def parse(self, query, field='', op='', **attrs):
         """Return lucene parsed Query.
         
@@ -128,13 +164,39 @@ class Searcher(object):
         :param op: default query operator ('or', 'and')
         :param attrs: additional attributes to set on the parser
         """
-        # parser's aren't thread-safe (nor slow), so create one each time
+        # parsers aren't thread-safe (nor slow), so create one each time
         parser = lucene.QueryParser(field, self.analyzer)
         if op:
             parser.defaultOperator = getattr(lucene.QueryParser.Operator, op.upper())
         for name, value in attrs.items():
             setattr(parser, name, value)
         return parser.parse(query)
+    def highlight(self, query, text, count=1, span=True, formatter=None, encoder=None, field='', **attrs):
+        """Return highlighted text fragments which match the query.
+        
+        :param query: query string or lucene Query
+        :param text: text string to be searched
+        :param count: maximum number of fragments
+        :param span: only highlight terms which would contribute to a hit
+        :param formatter: optional lucene Formatter
+        :param encoder: optional lucene Encoder
+        :param field: default query field name
+        :param attrs: additional attributes to set on the highlighter
+        """
+        if not isinstance(query, lucene.Query):
+            query = self.parse(query, field)
+        highlighter = lucene.Highlighter(*filter(None, [formatter, encoder, lucene.QueryScorer(query, field)]))
+        for name, value in attrs.items():
+            setattr(highlighter, name, value)
+        if lucene.VERSION <= '2.4.1': # memory leak in string array
+            tokens = self.analyzer.tokenStream(field, lucene.StringReader(text))
+            if span:
+                tokens = lucene.CachingTokenFilter(tokens)
+                highlighter.fragmentScorer = lucene.HighlighterSpanScorer(query, field, tokens)
+            return map(unicode, highlighter.getBestTextFragments(tokens, text, True, count))
+        if not span:
+            highlighter.fragmentScorer = lucene.QueryTermScorer(query, field)
+        return list(highlighter.getBestFragments(self.analyzer, field, text, count))
     def count(self, *query, **options):
         """Return number of hits for given query or term.
         
@@ -162,7 +224,7 @@ class Searcher(object):
             filter = Filter(filter)
         # use custom HitCollector if all results are necessary, otherwise let lucene's TopDocs handle it
         if count is None:
-            collector = HitCollector(self)
+            collector = HitCollector()
             super(Searcher, self).search(query, filter, collector)
             return Hits(self, *collector.sorted(key=sort, reverse=reverse))
         if sort is None:
@@ -181,9 +243,9 @@ class IndexSearcher(Searcher, lucene.IndexSearcher, IndexReader):
     """Inherited lucene IndexSearcher, with a mixed-in IndexReader.
     
     :param directory: directory path or lucene Directory
-    :param analyzer: lucene Analyzer class
+    :param analyzer: lucene Analyzer, default StandardAnalyzer
     """
-    def __init__(self, directory, analyzer=lucene.StandardAnalyzer):
+    def __init__(self, directory, analyzer=None):
         Searcher.__init__(self, directory, analyzer)
         self.filters = {}
     def facets(self, ids, *keys):
@@ -213,9 +275,9 @@ class MultiSearcher(Searcher, lucene.MultiSearcher):
     """Inherited lucene MultiSearcher.
     
     :param searchers: lucene.Searchers or directory
-    :param analyzer: lucene Analyzer class
+    :param analyzer: lucene Analyzer, default StandardAnalyzer
     """
-    def __init__(self, searchers, analyzer=lucene.StandardAnalyzer):
+    def __init__(self, searchers, analyzer=None):
         searchers = [searcher if isinstance(searcher, lucene.Searcher) else lucene.IndexSearcher(searcher) for searcher in searchers]
         Searcher.__init__(self, searchers, analyzer)
 
@@ -226,16 +288,23 @@ class IndexWriter(lucene.IndexWriter):
     """Inherited lucene IndexWriter.
     Supports setting fields parameters explicitly, so documents can be represented as dictionaries.
     
-    :param directory: directory path or lucene Directory
+    :param directory: directory path or lucene Directory, default RAMDirectory
     :param mode: file mode (rwa), except updating (+) is implied
-    :param analyzer: lucene Analyzer class
+    :param analyzer: lucene Analyzer, default StandardAnalyzer
+    :param mfl: MaxFieldLength, default IndexWriter.DEFAULT_MAX_FIELD_LENGTH
     """
     __len__ = lucene.IndexWriter.numDocs
     __del__ = Searcher.__del__.im_func
     parse = Searcher.parse.im_func
-    def __init__(self, directory=None, mode='a', analyzer=lucene.StandardAnalyzer):
-        create = [mode == 'w'] * (mode != 'a')
-        lucene.IndexWriter.__init__(self, directory or lucene.RAMDirectory(), analyzer(), *create)
+    def __init__(self, directory=None, mode='a', analyzer=None, mfl=10000):
+        if not isinstance(analyzer, (lucene.Analyzer, type(None))):
+            warnings.warn("Support for analyzer constructors will be removed.", DeprecationWarning)
+            analyzer = analyzer()
+        args = [(directory or lucene.RAMDirectory()), (lucene.StandardAnalyzer() if analyzer is None else analyzer)]
+        if mode != 'a':
+            args.append(mode == 'w')
+        args.append(mfl if isinstance(mfl, self.MaxFieldLength) else self.MaxFieldLength(mfl))
+        lucene.IndexWriter.__init__(self, *args)
         self.fields = {}
     @property
     def segments(self):
@@ -293,7 +362,7 @@ class Indexer(IndexWriter):
     """
     def __init__(self, *args, **kwargs):
         IndexWriter.__init__(self, *args, **kwargs)
-        self.indexSearcher = IndexSearcher(self.directory, self.getAnalyzer)
+        self.indexSearcher = IndexSearcher(self.directory, self.analyzer)
     def __getattr__(self, name):
         if name == 'indexSearcher':
             raise AttributeError(name)
@@ -308,4 +377,4 @@ class Indexer(IndexWriter):
         "Commit writes and refresh searcher.  Not thread-safe."
         IndexWriter.commit(self)
         if not self.current:
-            self.indexSearcher = IndexSearcher(self.directory, self.getAnalyzer)
+            self.indexSearcher = IndexSearcher(self.directory, self.analyzer)
