@@ -49,7 +49,8 @@ class Query(object):
     @classmethod
     def range(cls, name, start, stop, lower=True, upper=False):
         "Return lucene ConstantScoreRangeQuery, by default with a half-open interval."
-        return cls(lucene.ConstantScoreRangeQuery, name, start, stop, lower, upper)
+        base = lucene.TermRangeQuery if hasattr(lucene, 'TermRangeQuery') else lucene.ConstantScoreRangeQuery
+        return cls(base, name, start, stop, lower, upper)
     @classmethod
     def phrase(cls, name, *values):
         "Return lucene PhraseQuery.  None may be used as a placeholder."
@@ -124,8 +125,8 @@ class SpanQuery(Query):
 
 class HitCollector(lucene.PythonHitCollector):
     "Collect all ids and scores efficiently."
-    def __init__(self, searcher):
-        lucene.PythonHitCollector.__init__(self, searcher)
+    def __init__(self):
+        lucene.PythonHitCollector.__init__(self)
         self.collect = {}.__setitem__
     def sorted(self, key=None, reverse=False):
         "Return ordered ids and scores."
