@@ -2,6 +2,7 @@ import unittest
 import os, optparse
 import tempfile, shutil
 import itertools
+import warnings
 from datetime import date
 import lucene
 from lupyne import engine
@@ -21,12 +22,20 @@ class TestCase(BaseTest):
     
     def testInterface(self):
         "Indexer and document interfaces."
-        indexer = engine.Indexer()
+        assert engine.IndexWriter.__init__.im_func.func_defaults[-1] == lucene.IndexWriter.DEFAULT_MAX_FIELD_LENGTH
+        with warnings.catch_warnings(record=True) as deprecations:
+            writer = engine.IndexWriter(analyzer=lucene.StandardAnalyzer)
+            engine.IndexSearcher(writer.directory, analyzer=lucene.WhitespaceAnalyzer)
+        assert len(deprecations) == 2
+        Stemmer = engine.Analyzer(lucene.StandardAnalyzer(), lucene.PorterStemFilter, lucene.TypeAsPayloadTokenFilter)
+        indexer = engine.Indexer(analyzer=Stemmer)
         self.assertRaises(lucene.JavaError, engine.Indexer, indexer.directory)
         indexer.set('text')
-        indexer.set('name', store=True, index=False)
+        indexer.set('name', store=True, index=False, boost=2.0)
+        for field in indexer.fields['name'].items('sample'):
+            assert isinstance(field, lucene.Field) and field.boost == 2.0
         indexer.set('tag', store=True, index=True)
-        indexer.add(text='hello world', name='sample', tag=['python', 'search'])
+        indexer.add(text='hello worlds', name='sample', tag=['python', 'search'])
         assert len(indexer) == 1 and list(indexer) == []
         assert not indexer.optimized
         indexer.commit()
@@ -56,6 +65,7 @@ class TestCase(BaseTest):
         assert list(indexer.docs('text', 'hi')) == []
         assert list(indexer.docs('text', 'world', counts=True)) == [(0, 1)]
         assert list(indexer.positions('text', 'world')) == [(0, [1])]
+        assert list(indexer.positions('text', 'world', payloads=True)) == [(0, [(1, '<ALPHANUM>')])]
         hits = indexer.search('text:hello')
         assert len(hits) == hits.count == 1
         assert hits.ids == [0]
@@ -85,6 +95,9 @@ class TestCase(BaseTest):
         assert len(indexer) == 3
         x, y = engine.Filter([0, 1]).bits(), engine.Filter([1, 2]).bits()
         assert list(x & y) == [1] and list(x | y) == [0, 1, 2] and list(x -y) == [0]
+        indexer.add(text=lucene.WhitespaceTokenizer(lucene.StringReader('?')))
+        indexer.commit()
+        assert list(indexer.terms('text')) == ['?']
     
     def testBasic(self):
         "Text fields and simple searches."
@@ -173,6 +186,17 @@ class TestCase(BaseTest):
         near = queries[0].near(queries[1] | queries[2], slop=1)
         assert indexer.count(queries[0] - near) == count - 1
         assert indexer.count(queries[0][:100]) == count - 1
+        spans = dict(indexer.spans(queries[0]))
+        assert len(spans) == count and spans == dict(indexer.docs('text', 'persons', counts=True))
+        near = queries[0].near(queries[1], slop=2)
+        (id, positions), = indexer.spans(near, positions=True)
+        assert indexer[id]['amendment'] == '4' and positions == [(3, 6)]
+        with warnings.catch_warnings(record=True) as leaks:
+            assert 'persons' in indexer.termvector(id, 'text')
+            assert dict(indexer.termvector(id, 'text', counts=True))['persons'] == 2
+            assert dict(indexer.positionvector(id, 'text'))['persons'] == [3, 26]
+            assert dict(indexer.positionvector(id, 'text', offsets=True))['persons'] == [(46, 53), (301, 308)]
+        assert len(leaks) == 2 * (lucene.VERSION <= '2.4.1')
         del indexer
         assert engine.Indexer(self.tempdir)
     
@@ -220,7 +244,6 @@ class TestCase(BaseTest):
         count = indexer.count(field.prefix(lng[:3]))
         assert count > len(hits)
         assert count == indexer.count(field.range(lng[:3], lng[:3]+'~'))
-        self.assertRaises(lucene.JavaError, indexer.search, engine.Query.prefix(longitude, lng[:3]))
         assert count > indexer.count(engine.Query.term('state', 'CA'), filter=engine.Query.term(longitude, lng).filter())
         hits = indexer.search('zipcode:90*')
         (field, facets), = indexer.facets(hits.ids, 'state:county').items()
@@ -235,10 +258,14 @@ class TestCase(BaseTest):
         for name, params in fixture.zipcodes.fields.items():
             indexer.set(name, **params)
         field = indexer.fields['tile'] = engine.PointField('tile', precision=15, start=10, step=2, store=True)
+        points = []
         for doc in fixture.zipcodes.docs():
             if doc['state'] == 'CA':
                 lat, lng = doc.pop('latitude'), doc.pop('longitude')
                 indexer.add(doc, tile=[(lng, lat)], latitude=str(lat), longitude=str(lng))
+                if doc['city'] == 'Los Angeles':
+                    points.append((lng, lat))
+        assert len(list(engine.PolygonField('', precision=15).items(points))) > len(points)
         indexer.commit()
         city, zipcode, tile = 'Beverly Hills', '90210', '023012311120332'
         hit, = indexer.search('zipcode:' + zipcode)
@@ -267,9 +294,13 @@ class TestCase(BaseTest):
         indexer = engine.Indexer(self.tempdir)
         indexer.set('amendment', engine.FormatField, format='{0:02n}', store=True)
         indexer.set('date', engine.DateTimeField, store=True)
+        try:
+            indexer.set('size', engine.NumericField, store=True)
+        except AttributeError:
+            indexer.set('size', engine.FormatField, format='{0:04n}', store=True)
         for doc in fixture.constitution.docs():
             if 'amendment' in doc:
-                indexer.add(amendment=int(doc['amendment']), date=doc['date'])
+                indexer.add(amendment=int(doc['amendment']), date=doc['date'], size=len(doc['text']))
         indexer.commit()
         query = engine.Query.range('amendment', '', indexer.fields['amendment'].format(10))
         assert indexer.count(query) == 9
@@ -279,7 +310,8 @@ class TestCase(BaseTest):
         assert str(query) == 'date:Y:{ TO 1921} date:Ym:[1921 TO 1921-12]'
         assert indexer.count(query) == 19
         query = indexer.fields['date'].range(date(1919, 1, 1), date(1921, 12, 31))
-        fields = [lucene.ConstantScoreRangeQuery.cast_(clause.query).field for clause in query]
+        cls = lucene.TermRangeQuery if hasattr(lucene, 'TermRangeQuery') else lucene.ConstantScoreRangeQuery
+        fields = [cls.cast_(clause.query).field for clause in query]
         assert fields == ['date:Ymd', 'date:Ym', 'date:Y', 'date:Ym', 'date:Ymd']
         hits = indexer.search(query)
         assert [hit['amendment'] for hit in hits] == ['18', '19']
@@ -291,7 +323,42 @@ class TestCase(BaseTest):
         assert 0 < indexer.count(query) <= 12
         assert len(indexer.fields['date'].within(-100)) <= 3
         assert len(indexer.fields['date'].within(-100.0)) > 3
-        assert len(indexer.fields['date'].within(-100, 1)) > 3
+        assert len(indexer.fields['date'].within(-100, seconds=1, utc=True)) > 3
+        field, numeric = indexer.fields['size'], hasattr(engine, 'NumericField')
+        assert numeric == (len(list(indexer.terms('size'))) > len(indexer))
+        sizes = dict((id, int(indexer[id]['size'])) for id in indexer)
+        ids = sorted((id for id in sizes if sizes[id] >= 1000), key=sizes.get)
+        query = field.range(1000, None) if numeric else engine.Query.range('size', '1000', None)
+        hits = indexer.search(query, sort=sizes.get)
+        assert hits.ids == ids
+        hits = indexer.search(query, count=3, sort=lucene.SortField('size', lucene.SortField.LONG))
+        assert hits.ids == ids[:len(hits)]
+        query = field.range(None, 1000) if numeric else engine.Query.range('size', None, '1000')
+        assert indexer.count(query) == len(sizes) - len(ids)
+        if numeric:
+            self.assertRaises(OverflowError, list, field.items(-2**64))
+            nf, = field.items(0.5)
+            assert nf.numericValue.doubleValue() == 0.5
+            assert str(field.range(-2**64, 0)) == 'size:[* TO 0}'
+            assert str(field.range(0.5, None, upper=True)) == 'size:[0.5 TO *]'
+    
+    def testHighlighting(self):
+        "Highlighting text fragments."
+        indexer = engine.Indexer()
+        amendments = dict((doc['amendment'], doc['text']) for doc in fixture.constitution.docs() if 'amendment' in doc)
+        for amendment in amendments.values():
+            fragments = indexer.highlight('persons', amendment)
+            assert len(fragments) == ('persons' in amendment)
+            for fragment in fragments:
+                assert '<B>persons</B>' in fragment
+        text = amendments['4']
+        query = '"persons, houses, papers"'
+        fragments = indexer.highlight(query, text, count=3, span=False, formatter=lucene.SimpleHTMLFormatter('*', '*'))
+        assert len(fragments) == 2 and fragments[0].count('*') == 2*3 and '*persons*' in fragments[1]
+        fragment, = indexer.highlight(query, text, count=3, span=False, textFragmenter=lucene.SimpleFragmenter(200))
+        assert len(fragment) > len(text) and fragment.count('<B>persons</B>') == 2
+        fragment, = indexer.highlight(query, text, count=3)
+        assert len(fragment) < len(text) and fragment.count('<B>') == 3
 
 if __name__ == '__main__':
     lucene.initVM(lucene.CLASSPATH)
